@@ -1,7 +1,7 @@
 # -*- coding: UTF-8 -*-
 """
 Bug test cases
-@author: Aurélien Gâteau <aurelien.gateau@free.fr>
+@author: Aurélien Gâteau <mail@agateau.com>
 @license: GPL v3 or later
 """
 
@@ -11,13 +11,16 @@ import testutils
 
 from yokadi.ycli import tui
 from yokadi.ycli.main import YokadiCmd
-from yokadi.core.db import Task
+from yokadi.core import db, dbutils
+from yokadi.core.db import Task, setDefaultConfig
 from yokadi.core.yokadiexception import YokadiException
 
 
 class BugTestCase(unittest.TestCase):
     def setUp(self):
-        testutils.clearDatabase()
+        db.connectDatabase("", memoryDatabase=True)
+        self.session = db.getSession()
+        setDefaultConfig()
         tui.clearInputAnswers()
         self.cmd = YokadiCmd()
 
@@ -28,16 +31,30 @@ class BugTestCase(unittest.TestCase):
         tui.addInputAnswers("n")
         self.cmd.do_bug_add("notExistingProject newBug")
 
-        tasks = list(Task.select())
+        tasks = self.session.query(Task).all()
         result = [x.title for x in tasks]
-        expected = [u"t1"]
+        expected = ["t1"]
         self.assertEqual(result, expected)
 
-        kwDict = Task.get(1).getKeywordDict()
+        kwDict = self.session.query(Task).get(1).getKeywordDict()
         self.assertEqual(kwDict, dict(_severity=2, _likelihood=4, _bug=123))
 
         for bad_input in ("",  # No project
                           "x"):  # No task name
             self.assertRaises(YokadiException, self.cmd.do_bug_add, bad_input)
 
+    def testEdit(self):
+        task = dbutils.addTask("prj", "bug", interactive=False)
+        kwDict = dict(_severity=1, _likelihood=2, _bug=3)
+        task.setKeywordDict(kwDict)
+        self.session.commit()
+
+        tui.addInputAnswers("bug edited", "2", "4", "6")
+        self.cmd.do_bug_edit(str(task.id))
+
+        task = dbutils.getTaskFromId(task.id)
+        self.assertEqual(task.title, "bug edited")
+        kwDict = task.getKeywordDict()
+        self.assertEqual(kwDict, dict(_severity=2, _likelihood=4, _bug=6))
+
 # vi: ts=4 sw=4 et
