@@ -1,7 +1,7 @@
 # -*- coding: UTF-8 -*-
 """
 Parser utilities test cases
-@author: Aurélien Gâteau <aurelien.gateau@free.fr>
+@author: Aurélien Gâteau <mail@agateau.com>
 @license: GPL v3 or later
 """
 
@@ -9,10 +9,10 @@ import unittest
 from yokadi.ycli import parseutils
 
 gTaskLineToParsedStructList = [
-    (u"project some text @keyword1 @keyword2=12 some other text", (u"project", u"some text some other text", {u"keyword1":None, u"keyword2":12})),
-    (u"project ééé", (u"project", u"ééé", {})),
-    (u"project let's include quotes\"", (u"project", u"let's include quotes\"", {})),
-    (u"   project this  one has  extra spaces  ", (u"project", u"this one has extra spaces", {})),
+    ("project some text @keyword1 @keyword2=12 some other text", ("project", "some text some other text", {"keyword1":None, "keyword2":12})),
+    ("project ééé", ("project", "ééé", {})),
+    ("project let's include quotes\"", ("project", "let's include quotes\"", {})),
+    ("   project this  one has  extra spaces  ", ("project", "this one has extra spaces", {})),
     ]
 
 
